import * as React from 'react';
import { Connection } from '../../Connection';
import { Types, Maybe } from '../../common';
import {
  State as AppState,
  Update as AppUpdate,
  StateSettings,
} from '../../state';
import { getHashData } from '../../utils';
import { Header } from './';
import { List, WorldMap, Stats } from '../';
import { Persistent, PersistentObject, PersistentSet } from '../../persist';

import './Chain.css';

export type ChainDisplay = 'list' | 'map' | 'settings' | 'consensus' | 'stats';

interface ChainProps {
  appState: Readonly<AppState>;
  appUpdate: AppUpdate;
  connection: Promise<Connection>;
  settings: PersistentObject<StateSettings>;
  pins: PersistentSet<Types.NodeName>;
  sortBy: Persistent<Maybe<number>>;
}

interface ChainState {
  display: ChainDisplay;
}

export class Chain extends React.Component<ChainProps, ChainState> {
  constructor(props: ChainProps) {
    super(props);

    let display: ChainDisplay = 'map';

    switch (getHashData().tab) {
      case 'list':
        display = 'list';
        break;
      case 'settings':
        display = 'settings';
        break;
    }

    this.state = {
      display,
    };
  }

  public render() {
    const { appState } = this.props;
    const { best, finalized, blockTimestamp, blockAverage, nodes } = appState;
    const { display: currentTab } = this.state;

    return (
      <div className="Chain">
        <Header
          nodesLength={nodes.sorted().length}
          best={best}
          finalized={finalized}
          blockAverage={blockAverage}
          blockTimestamp={blockTimestamp}
          currentTab={currentTab}
          setDisplay={this.setDisplay}
        />
        <div className="Chain-content-container">
          <div className="Chain-content">{this.renderContent()}</div>
        </div>
        <div className="Chain-line-separator">
          GHOST TestNet is Now Live. Check out the&nbsp;
          <a onClick={()=> window.open("https://blog.ghostchain.io/ghost-chain-startup-guide/", "_blank")}>
            Quick Startup Guide
          </a>.
        </div>
      </div>
    );
  }

  private renderContent() {
    const { display } = this.state;

    const { appState, appUpdate, pins, sortBy } = this.props;

    if (display === 'list') {
      return (
        <List
          appState={appState}
          appUpdate={appUpdate}
          pins={pins}
          sortBy={sortBy}
        />
      );
    }

    if (display === 'map') {
      return <WorldMap appState={appState} />;
    }

    if (display === 'stats') {
      return <Stats appState={appState} />;
    }

    throw new Error('invalid `display`: ${display}');
  }

  private setDisplay = (display: ChainDisplay) => {
    this.setState({ display });
  };
}
