import * as React from 'react';
import { Types, Maybe } from '../../common';
import { formatNumber, secondsWithPrecision } from '../../utils';
import { Tab, ChainDisplay } from './';
import { Tile, Ago, Icon } from '../';

import blockIcon from '../../icons/cube.svg';
import finalizedIcon from '../../icons/cube-alt.svg';
import blockTimeIcon from '../../icons/history.svg';
import lastTimeIcon from '../../icons/watch.svg';
import listIcon from '../../icons/list-alt-regular.svg';
import worldIcon from '../../icons/location.svg';
import settingsIcon from '../../icons/settings.svg';
import statsIcon from '../../icons/graph.svg';
import ghostLogoIcon from '../../icons/ghost-logo.svg';

import './Header.css';

interface HeaderProps {
  nodesLength: number;
  best: Types.BlockNumber;
  finalized: Types.BlockNumber;
  blockTimestamp: Types.Timestamp;
  blockAverage: Maybe<Types.Milliseconds>;
  currentTab: ChainDisplay;
  setDisplay: (display: ChainDisplay) => void;
}

export class Header extends React.Component<HeaderProps> {
  public shouldComponentUpdate(nextProps: HeaderProps) {
    return (
      this.props.nodesLength !== nextProps.nodesLength ||
      this.props.best !== nextProps.best ||
      this.props.finalized !== nextProps.finalized ||
      this.props.blockTimestamp !== nextProps.blockTimestamp ||
      this.props.blockAverage !== nextProps.blockAverage ||
      this.props.currentTab !== nextProps.currentTab
    );
  }

  public render() {
    const { nodesLength, best, finalized, blockTimestamp, blockAverage } = this.props;
    const { currentTab, setDisplay } = this.props;

    return (
      <div className="Header">
        <div className="Header-container">
          <div className="Header-logo" onClick={()=> window.open("https://ghostchain.io/", "_blank")}>
            <Icon src={ghostLogoIcon} />
          </div>
          <div className="Header-data">
            <Tile title="Latest Block">
              #{formatNumber(best)}
            </Tile>
            <Tile title="Finalized Block">
              #{formatNumber(finalized)}
            </Tile>
            <Tile title="Average Time">
              {blockAverage == null
                ? '-'
                : secondsWithPrecision(blockAverage / 1000)}
            </Tile>
            <Tile title="Last Block">
              <Ago when={blockTimestamp} />
            </Tile>
            <Tile title="Nodes">
              {nodesLength}
            </Tile>
          </div>
          <div className="Header-tabs">
            <Tab
              icon={worldIcon}
              label="Map"
              display="map"
              tab="map"
              current={currentTab}
              setDisplay={setDisplay}
            />
            <Tab
              icon={listIcon}
              label="List"
              display="list"
              tab=""
              current={currentTab}
              setDisplay={setDisplay}
            />
            <Tab
              icon={statsIcon}
              label="Stats"
              display="stats"
              tab="stats"
              current={currentTab}
              setDisplay={setDisplay}
            />
          </div>
        </div>
      </div>
    );
  }
}
